@@ -192,6 +192,7 @@
 #define HAVE_PTHREAD_ATTR_SETSCOPE 1
 #define HAVE_PTHREAD_ATTR_SETSTACKSIZE 1
 /* #undef HAVE_PTHREAD_CONDATTR_CREATE */
+/* #undef HAVE_PTHREAD_GETAFFINITY_NP */
 #define HAVE_PTHREAD_KEY_DELETE 1
 /* #undef HAVE_PTHREAD_KILL */
 #define HAVE_PTHREAD_RWLOCK_RDLOCK 1
@@ -277,10 +278,10 @@
   #define SIZEOF_SIZE_T  SIZEOF_LONG
 #else
 /* No indentation, to fetch the lines from verification scripts */
-#define SIZEOF_LONG   4
-#define SIZEOF_VOIDP  4
-#define SIZEOF_CHARP  4
-#define SIZEOF_SIZE_T 4
+#define SIZEOF_LONG   8
+#define SIZEOF_VOIDP  8
+#define SIZEOF_CHARP  8
+#define SIZEOF_SIZE_T 8
 #endif
 
 #define HAVE_LONG 1
@@ -295,7 +296,7 @@
 /* #undef HAVE_UCHAR */
 #define SIZEOF_UINT 4
 #define HAVE_UINT 1
-#define SIZEOF_ULONG 4
+#define SIZEOF_ULONG 8
 #define HAVE_ULONG 1
 /* #undef SIZEOF_INT8 */
 /* #undef HAVE_INT8 */
@@ -333,7 +334,7 @@
     #define WORDS_BIGENDIAN 1
   #endif
 #else
-#define WORDS_BIGENDIAN 1 
+/* #undef WORDS_BIGENDIAN */
 #endif
 
 /* Define to `__inline__' or `__inline' if that's what the C compiler calls
@@ -397,8 +398,8 @@
 #define STACK_DIRECTION -1
 
 #define SYSTEM_TYPE "OpenBSD"
-#define MACHINE_TYPE "powerpc"
-#define DEFAULT_MACHINE "powerpc"
+#define MACHINE_TYPE "aarch64"
+#define DEFAULT_MACHINE "aarch64"
 /* #undef HAVE_DTRACE */
 
 #define SIGNAL_WITH_VIO_CLOSE 1
@@ -534,16 +535,16 @@
 
 #define MYSQL_VERSION_MAJOR 10
 #define MYSQL_VERSION_MINOR 4
-#define MYSQL_VERSION_PATCH 12
+#define MYSQL_VERSION_PATCH 13
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 10.4.12"
+#define PACKAGE_STRING "MySQL Server 10.4.13"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "10.4.12"
-#define VERSION "10.4.12"
+#define PACKAGE_VERSION "10.4.13"
+#define VERSION "10.4.13"
 #define PROTOCOL_VERSION 10
 
 #define MALLOC_LIBRARY "system"
